(* glFramebuffer.mli    Framebuffer object extension of OpenGL ES
 *
 * http://khronos.org/registry/gles/extensions/OES/OES_framebuffer_object.txt
 *)
type renderbuffer_id

(* Reserved renderbuffer id 0, representing no renderbuffer.
 *)
val renderbuffer0 : renderbuffer_id

type rendertarget = [ `renderbuffer ]

type renderfmt =
    (* For color; last 2 are optional (not available in all implementations) *)
    [ `rgb565 | `rgba4 | `rgb5_a1 | `rgba8 | `rgb8
    (* For depth; last 2 are optional *)
    | `depth_component16 | `depth_component24 | `depth_component32
    (* For stencil; all are optional *)
    | `stencil_index1 | `stencil_index4 | `stencil_index8
    ]

type renderparam =
    [ `width | `height | `red_size | `green_size | `blue_size
    | `alpha_size | `depth_size | `stencil_size
    ]
    
type renderparam_fmt = [ `internal_format ]

type id (* Framebuffer id *)

(* Reserved framebuffer id 0, representing no framebuffer object.
 *)
val framebuffer0 : id

type target = [ `framebuffer ]

type status =
    [ `complete | `incomplete_attachment | `incomplete_missing_attachment
    | `incomplete_dimensions | `incomplete_formats | `unsupported
    ]
    
type attachment = [ `color0 | `depth | `stencil ]

type textarget = [ `texture_2d ]

type attachtype =
    [ `none
    | `renderbuffer of renderbuffer_id
    | `texture of GlTex.texture_id
    ]

type enumeration =
    [ `renderbuffer ]

val is_renderbuffer : renderbuffer_id -> bool
val bind_renderbuffer : rendertarget -> renderbuffer_id -> unit
val delete_renderbuffer : renderbuffer_id -> unit
val delete_renderbuffers : renderbuffer_id array -> unit
val gen_renderbuffer : unit -> renderbuffer_id
val gen_renderbuffers : int -> renderbuffer_id array
val renderbuffer_storage : rendertarget -> renderfmt -> int -> int -> unit
val get_renderbuffer_parameter : rendertarget -> renderparam -> int
val get_renderbuffer_parameter_fmt :
    rendertarget -> renderparam_fmt -> renderfmt
val is_framebuffer : id -> bool
val bind_framebuffer : target -> id -> unit
val delete_framebuffer : id -> unit
val delete_framebuffers : id array -> unit
val gen_framebuffer : unit -> id
val gen_framebuffers : int -> id array
val check_status : target -> status
val renderbuffer :
    target -> attachment -> rendertarget -> renderbuffer_id -> unit
val texture2d :
    target -> attachment -> textarget -> GlTex.texture_id -> int -> unit
val get_attachment_type : target -> attachment -> attachtype
val generate_mipmap : textarget -> unit
val code : enumeration -> int
